extern void MakeLog(Widget,XtPointer,XtPointer);

extern void initusefile();
extern void SaveUsefile();
extern void SetUseKanji(int kpos, int val);
extern int NumInUsefile();
extern int InUsefile(int kindex);

extern void initlog();
