extern void MakeGradeSelect(),CountKanji();
extern Widget gradelevelForm;
extern Widget gradeButtons[8];
extern void printgrades(), parsegrades(char *);
extern void GradeCallback(Widget,XtPointer,XtPointer);

