void InitRadicals();
int HaveRadicals();
void MakeRadicalinputPopup();
void ShowRadicalinput(Widget w,XtPointer client_data, XtPointer call_data);

void FindRadicals(XChar2b kanji, XChar2b *radlist);
