extern int nextwork(unsigned char **);
extern unsigned char *nextchar(unsigned char *c);
extern void readstructs();
extern void readedict();

extern short skipfromthree(int one, int two, int three);

extern int trans_to_index(TRANSLATION trans);
